

#include "vdis.h"

#define THREE_DEG_SQR_2_RAD 0.15707963267948966192313216916398

dispost_office_type disiface;


/* **************************************************************
************************************************************** */
dis_data::dis_data(basic_platform *x) : platform_data(x) {

   next_update_time = update_time = -1.0;
}


/* **************************************************************
************************************************************** */
dis_data::~dis_data() {

}


/* **************************************************************
************************************************************** */
void dis_data::label(dbl_llist_manager *label_manager) {


}


/* **************************************************************
************************************************************** */
void dis_data::update(double *machine_time) {

   vector3d local, world;
   vector3d world_vel, world_accel;
   vector3d diff, ediff;
   vector3d localypr, worldypr;
   double t;
   int manditory_flag;

   manditory_flag = (parent->platform_flag & PLATFORM_FLAG_FORCED_UPDATE) || next_update_time < *machine_time;

   if (!(parent->platform_flag & PLATFORM_FLAG_UPDATE) && !manditory_flag)
      return;

   copyarray3(local, parent->plat_clone.pos);
   local2dis(local, world, DIS_POSITION);

   localypr[0] = parent->plat_clone.rpy[2];
   localypr[1] = parent->plat_clone.rpy[1];
   localypr[2] = parent->plat_clone.rpy[0];
   local2disypr(localypr, worldypr);

   subeqarray3(diff, es.location, world);
   subeqarray3(ediff, es.ypr, worldypr);
   
   if (manditory_flag || dotproduct3(diff, diff) > 1.0 || dotproduct3(ediff, ediff) > THREE_DEG_SQR_2_RAD) {

      copyarray3(local, parent->plat_clone.vel);
      local2dis(local, world_vel, DIS_VECTOR);

      copyarray3(local, parent->plat_clone.acc);
      local2dis(local, world_accel, DIS_VECTOR);
   
      es.dr_ypr_velocity[0] = (float)-parent->plat_clone.drpy[2];
      es.dr_ypr_velocity[1] = (float)parent->plat_clone.drpy[1];
      es.dr_ypr_velocity[2] = (float)parent->plat_clone.drpy[0];

      copyarray3(es.location, world);
      copyarray3(es.velocity, (float)world_vel);
      copyarray3(es.dr_accel, (float)world_accel);

      copyarray3(es.ypr, (float)worldypr);
//asdf accel

      t = parent->plat_clone.timer;
      es.header.time = distime(&t);

      em.event_entity_id = disiface.dis_event_counter;
      disiface.dis_event_counter++;
      disiface.dis_event_counter &= 0xffff;
      if (!disiface.dis_event_counter)
         disiface.dis_event_counter = 1;

      em.header.time = distime(&t);

      next_update_time = *machine_time + update_time;
      flags |= DIS_FLAG_UPDATE;
   }

   parent->platform_flag &= ~PLATFORM_FLAG_FORCED_UPDATE;
}


/* **************************************************************
************************************************************** */
void dis_data::render(iface *isim) {

   if ((parent->platform_flag & (PLATFORM_FLAG_CONTROL | PLATFORM_FLAG_EXTERNAL)) || !(flags & DIS_FLAG_UPDATE))
      return;

   es.build(&send_buffer);
   disiface.send_message(&send_buffer);

   parent->perform(isim, PERFORM_UPDATE_SENSORS, NULL);
   if (em.systemman.head) {
      em.build(&send_buffer);
      disiface.send_message(&send_buffer);
   }

   flags &= ~DIS_FLAG_UPDATE;
}


/* **************************************************************
************************************************************** */
int dis_data::perform(iface *isim, int task, void *data) {

   buffer_type buffer;
   basic_radar_data_type *brdt;
   dis_em_systems_type *ptr;
   DIS_emissions_systems_type *petr;
   DIS_emissions_beams_type *pebtr;

   switch (task) {
      case PERFORM_PLATFORM_DEACTIVATE:

         if ((parent->platform_flag & (PLATFORM_FLAG_CONTROL | PLATFORM_FLAG_EXTERNAL)))
            break;

         es.appearance_flag |= DIS_PDU_APPEARANCE_DESTROYED;
         es.build(&buffer);
         disiface.send_message(&buffer);
         return 1;

      case PERFORM_PLATFORM_INIT_EMITTER:
         sensorman.dest();
         em.systemman.dest();
         return 1;

      case PERFORM_PLATFORM_EMITTER:
         brdt = (basic_radar_data_type *)data;
         for (ptr = (dis_em_systems_type *)sensorman.head; ptr && ptr->id != brdt->id; ptr = (dis_em_systems_type *)ptr->next);

         if (!ptr) {
            sensorman.append(ptr = new dis_em_systems_type, NULL);
            ptr->id = brdt->id;

            // * emitter dis pdu data setup here *
            em.systemman.append(petr = new DIS_emissions_systems_type, NULL);
            petr->emitter_id = brdt->id;

            if (brdt->str)
               petr->emitter_name = atoi(brdt->str->sym_sim.string);

            petr->beamman.append(ptr->es_radar = pebtr = new DIS_emissions_beams_type, NULL);

            pebtr->beam_id = 1;
            pebtr->beam_parameter_index = 0;
            pebtr->freq_range = 0;
            pebtr->beam_sweep_sync = 0;

            pebtr->high_density = DIS_TRACK_JAM_SELECTED;
            pebtr->jamming_mode = 0;
         }

         else
            pebtr = ptr->es_radar;

         pebtr->frequency = (float)brdt->frequency;
         pebtr->power = (float)brdt->power;
         pebtr->prf = (float)brdt->prf;
         pebtr->pulse_width = (float)brdt->pulse_width;
         pebtr->beam_azimuth = (float)brdt->beam_azimuth;
         pebtr->beam_az_sweep = (float)brdt->beam_az_sweep;
         pebtr->beam_elevation = (float)brdt->beam_elevation;
         pebtr->beam_ele_sweep = (float)brdt->beam_ele_sweep;

         switch (brdt->mode) {
            case RADAR_MODE_SEARCH:
               pebtr->beam_function = DIS_BEAM_FUNCTION_SEARCH;
               break;

            case RADAR_MODE_AQUISITION:
               pebtr->beam_function = DIS_BEAM_FUNCTION_AQUISITION;
               break;

            case RADAR_MODE_TRACK:
               pebtr->beam_function = DIS_BEAM_FUNCTION_TRACK;
               break;

            case RADAR_MODE_GUIDANCE:
               pebtr->beam_function = DIS_BEAM_FUNCTION_GUIDANCE;
               break;

            default:
               pebtr->beam_function = DIS_BEAM_FUNCTION_OTHER;
               break;
         }

         return 1;

      default:
         break;
   }

   return platform_data::perform(isim, task, data);
}


/* **************************************************************
************************************************************** */
int dis_data::init(iface *isim, correlate *basis, symbol_correlation_type *str) {

   DIS_entity_type_type type;
   int t[8];
   
   sscanf(str->sym_sim.string, "%d:%d:%d:%d:%d:%d:%d:%d", &t[0], &t[1], &t[2], &t[3], &t[4], &t[5], &t[6], &t[7]);
   type.kind = t[0];
   type.domain =  t[1];
   type.country = t[2];
   type.category = t[3];
   type.subcategory = t[4];
   type.specific = t[5];
   type.extra = t[6];

   es.setup(parent->id,
      type.country == DIS_COUNTRY_USA ? DIS_FORCE_FRIENDLY : DIS_FORCE_OPPOSING,
      &type, DIS_DR_RVW);

   es.header.time = 0;

   es.velocity[0] = 0;
   es.velocity[1] = 0;
   es.velocity[2] = 0;

   es.location[0] = 0;
   es.location[1] = 0;
   es.location[2] = 0;

   es.ypr[0] = 0;
   es.ypr[1] = 0;
   es.ypr[2] = 0;

   es.dr_accel[0] = 0;
   es.dr_accel[1] = 0;
   es.dr_accel[2] = 0;

   es.dr_ypr_velocity[0] = 0;
   es.dr_ypr_velocity[1] = 0;
   es.dr_ypr_velocity[2] = 0;

   em.setup(parent->id);

   sensorman.dest();

   update_time = 4.0 + 0.01*(parent->id % 100);  // 5 seconds is the default DIS update time

   return 1;
}
