


/* ***********************************************************************
                Naval Air Warfare Center / Aircraft Division              
                        Patuxent Naval Air Station                         
 ========================================================================

        File:           platform.c                                         
                                                                        
        Classification: UNCLASSIFIED                                   
                                                                        
        Description:    This file contains the functions, methods, and
			global variables needed to manage the objects
			being visualized.
			
        Written by:	Alex Harper
*********************************************************************** */


#include "vissim.h"


/* ============================================================================
 ============================================================================ */
plat_stat_type::plat_stat_type() {

   pos[0] = pos[1] = pos[2] = 0;
   acc[0] = acc[1] = acc[2] = 0;
   vel[0] = vel[1] = vel[2] = 0;
   rpy[0] = rpy[1] = rpy[2] = 0;
   drpy[0] = drpy[1] = drpy[2] = 0;
   dead_recogn_flag = 0;

   init_pos[0] = init_pos[1] = init_pos[2] = 0;
   init_vel[0] = init_vel[1] = init_vel[2] = 0;
   elapsed_time = 0;

   machine_time = timer = -1.0;
}


/* ============================================================================
  dead recogning using system clock instead of sweg clock (sweg falling behind...)
 ============================================================================ */
void plat_stat_type::dead_recog(double *currtime, double *multiplier, double *newtime) {
      
   double speedscale;
   double t;

   if (machine_time > *newtime)
      return;

   speedscale = (*newtime-machine_time) * *multiplier;

   elapsed_time += speedscale;

   vel[0] += acc[0] * speedscale;
   vel[1] += acc[1] * speedscale;
   vel[2] += acc[2] * speedscale;

   t = 0.5 * elapsed_time;

   pos[0] = init_pos[0] + elapsed_time * (init_vel[0] + t * acc[0]);
   pos[1] = init_pos[1] + elapsed_time * (init_vel[1] + t * acc[1]);
   pos[2] = init_pos[2] + elapsed_time * (init_vel[2] + t * acc[2]);

   rpy[0] += drpy[0] * speedscale;
   rpy[1] += drpy[1] * speedscale;
   rpy[2] += drpy[2] * speedscale;

   machine_time = *newtime;
}


/* ============================================================================
 ============================================================================ */
int platform_data::perform(iface *isim, int task, void *data) {

   switch (task) {
      case PERFORM_QUERY_ID:
         *(int *)data = parent->id;
         return 1;

      default:
         break;
   }

   return 0;
}


/* ============================================================================
 ============================================================================ */
int basic_platform::perform(iface *isim, int task, void *data) {

   switch (task) {

      case PERFORM_PLATFORM_DEACTIVATE:
         if (vis_data)
            vis_data->perform(isim, task, data);

         return 1;

      default:
         break;
   }

   return 0;
}


/* ============================================================================
  generate a label for the current platform
 ============================================================================ */
void basic_platform::label(dbl_llist_manager *label_manager) {

   if (vis_data &&
       (platform_flag & PLATFORM_FLAG_VISIBLE) &&
       !(platform_flag & PLATFORM_FLAG_DEAD))
      ((platform_data *)vis_data)->label(label_manager);
}

/* ============================================================================
  Update position of a platform and/or the camera
  ============================================================================ */
void basic_platform::update(iface *isim, double *currtime, double *multiplier, double *machine_time, int pause) {

   if (!vis_data || (platform_flag & PLATFORM_FLAG_DEAD))
      return;

   platform_flag &= ~PLATFORM_FLAG_UPDATE;

   if (!pause) {
      update_data(machine_time);

      if ((platform_flag & PLATFORM_FLAG_DEAD_RECOG) && plat_clone.dead_recogn_flag) {
         plat_clone.dead_recog(currtime, multiplier, machine_time);
         platform_flag |= PLATFORM_FLAG_MOVEMENT;
      }

   }
   
   vis_data->update(machine_time);
   vis_data->render(isim);
}
